(* ::Package:: *)

(* :Name: FunctionsFor2DSymmetry` *)

(* :Title: Functions to create symmetric two-dimensional patterns *)

(* :Author: Karel Houfek *)

BeginPackage["FunctionsFor2DSymmetry`"];



Unprotect[
  CreateImageFromPalette,
  CreateRosetteByAveraging,
  CreateFriezeByCopying,
  CreateWallpaperOfGivenSymmetry,
  MorphTwoPatternsOfGivenSymmetry
];

Begin["`Private`"];

Clear[
  CreateImageFromPalette,
  CreateRosetteByAveraging,
  CreateFriezeByCopying,
  CreateWallpaperOfGivenSymmetry,
  MorphTwoPatternsOfGivenSymmetry
];

(* Basic routine to create a picture according to function[z] from a color map given as 
   rgbcolors (Mathematica imports any bitmap in this format using Import function) *)
CreateImageFromPalette[rgbcolors_, colorsrange_, function_, frange_, cellresolution_] := 
Module[{i, j, ip, jp, mp, np, x, y, z, fz, hxp, hyp, hx, hy, rgbcell},
  {np, mp} = Dimensions[rgbcolors];
  rgbcell = ConstantArray[0, {cellresolution[[2]], cellresolution[[1]]}];
  hxp = (colorsrange[[2,1]] - colorsrange[[1,1]]) / mp;
  hyp = (colorsrange[[2,2]] - colorsrange[[1,2]])/ np;
  hx = (frange[[2,1]] - frange[[1,1]])/ cellresolution[[1]];
  hy = (frange[[2,2]] - frange[[1,2]])/ cellresolution[[2]];
  Do[
    Do[
      z = frange[[1,1]] + (i - 0.5) * hx + I * (frange[[1,2]] + (j - 0.5) * hy);
      fz = function[z];
      {x, y} = {Re[fz], Im[fz]};
      If[
        x <= colorsrange[[1,1]] || x > colorsrange[[2,1]] || y <= colorsrange[[1,2]] || y > colorsrange[[2,2]],
        rgbcell[[j, i]] = RGBColor["Black"],
        ip = Ceiling[(x - colorsrange[[1,1]]) / hxp];
        jp = Ceiling[(y - colorsrange[[1,2]]) / hyp];
        rgbcell[[j,i]] = rgbcolors[[jp, ip]]
      ],
      {j, 1, cellresolution[[2]]}
    ],
    {i, 1, cellresolution[[1]]}
  ];
  Return[Image[rgbcell]];
];

(* This routine creates a rosette with a given symmetry point group from a picture in rgbcolors *)
CreateRosetteByAveraging[rgbcolors_, colorsrange_, function_, frange_, group_, degree_, cellresolution_] := 
Module[{i, j, ip, jp, mp, np, exps, x, y, z, g, gz, hxp, hyp, hx, hy, rgbcell},
  {np, mp} = Dimensions[rgbcolors];
  exps = Table[N[Exp[I*2*\[Pi]*i/degree]], {i, 1, degree}];
  Switch[
    group,
    "C",
    g[z_] := Sum[function[exps[[i]]*z], {i, 1, degree}] / degree,
    "D",
    g[z_] := Sum[function[exps[[i]]*z]+function[Conjugate[exps[[i]]*z]], {i, 1, degree}] / (2 * degree)
  ];
  Return[CreateImageFromPalette[rgbcolors, colorsrange, g, frange, cellresolution]];
];

(* Routine to create pictures with symmetry of given wallpaper groups
   It actually returns an array of triples 
   {"name of the group", "picture representing the primitive cell", "picture with this symmetry"} *)
CreateWallpaperOfGivenSymmetry[groups_, latticeangle_, latticelength_, frequencies_, amplitudes_, palettefile_, paletterange_, wallpaperrange_, wallpaperresolution_, magnification_] := Module[
  {rgbcolors, images, wave, wave3, wave4, i, ig, a, b, \[Alpha], X, Y, BF, coef, g},
  rgbcolors = Import[palettefile, "RGBColorArray"];
  images = {};
  wave[n_, z_] := Exp[2 * \[Pi] * I * (n[[1]] * X[z] + n[[2]] * Y[z])];
  wave3[n_, z_] := wave[n, z] + wave[{-n[[1]]-n[[2]], n[[1]]}, z] + wave[{n[[2]], -n[[1]]-n[[2]]}, z];
  wave4[n_, z_] := wave[n, z] + wave[{n[[2]], -n[[1]]}, z] + wave[-n, z] + wave[{-n[[2]], n[[1]]}, z];
  Do[
    {a, b} = latticelength;
    \[Alpha] = latticeangle;
    Switch[
      groups[[ig]],
      "p1",
      X[z_] := (Re[z] - Im[z] * Cot[\[Alpha]]) / a;
      Y[z_] := Im[z] / (b * Sin[\[Alpha]]);
      BF[n_, z_] := wave[n, z];
      coef = 1,
      "p2",
      X[z_] := (Re[z] - Im[z] * Cot[\[Alpha]]) / a;
      Y[z_] := Im[z] / (b * Sin[\[Alpha]]);
      BF[n_, z_] := wave[n, z] + wave[-n, z];
      coef = 1,
      "cm",
      a = 2 * latticelength[[1]] * Cos[\[Alpha]/2];
      b = latticelength[[1]] * Sin[\[Alpha]/2];
      X[z_] := Re[z] / a + Im[z] * a / (2 * b);
      Y[z_] := Re[z] / a - Im[z] * a / (2 * b);
      BF[n_, z_] := wave[n, z] + wave[{n[[2]], n[[1]]}, z];
      coef = 1/2,
      "cmm",
      a = 2 * latticelength[[1]] * Cos[\[Alpha]/2];
      b = latticelength[[1]] * Sin[\[Alpha]/2];
      X[z_] := Re[z] / a + Im[z] * a / (2 * b);
      Y[z_] := Re[z] / a - Im[z] * a / (2 * b);
      BF[n_, z_] := wave[n, z] + wave[{n[[2]], n[[1]]}, z] + wave[-n, z] + wave[{-n[[2]], -n[[1]]}, z];
      coef = 1/2,
      "pm",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + wave[{n[[1]], -n[[2]]}, z];
      coef = 1/2,
      "pg",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + (-1)^n[[1]] * wave[{n[[1]], -n[[2]]}, z];
      coef = 1/2,
      "pmm",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + wave[{n[[1]], -n[[2]]}, z] + wave[-n, z] + wave[{-n[[1]], n[[2]]}, z];
      coef = 1/2,
      "pmg",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + (-1)^n[[1]] * wave[{n[[1]], -n[[2]]}, z] + wave[-n, z] + (-1)^n[[1]] * wave[{-n[[1]], n[[2]]}, z];
      coef = 1/2,
      "pgg",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + (-1)^(n[[1]] + n[[2]]) * wave[{n[[1]], -n[[2]]}, z] + wave[-n, z] + (-1)^(n[[1]] + n[[2]]) * wave[{-n[[1]], n[[2]]}, z];
      coef = 1/2,
      "p4",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / a;
      BF[n_, z_] := wave4[n, z];
      coef = 1/2,
      "p4m",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / a;
      BF[n_, z_] := wave4[n, z] + wave4[{n[[1]], -n[[2]]}, z];
      coef = 1/4,
      "p4g",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / a;
      BF[n_, z_] := wave4[n, z] + (-1)^(n[[1]] + n[[2]]) * wave4[{n[[2]], n[[1]]}, z];
      coef = 1/4,
      "p3",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z];
      coef = 1/3,
      "p31m",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z] + wave3[{n[[2]], n[[1]]}, z];
      coef = 1/4,
      "p3m1",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z] + wave3[{-n[[2]], -n[[1]]}, z];
      coef = 1/4,
      "p6",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z] + wave3[-n, z];
      coef = 1/4,
      "p6m",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z] + wave3[{n[[2]], n[[1]]}, z] + wave3[-n, z] + wave3[{-n[[2]], -n[[1]]}, z];
      coef = 1/6
    ];
    g[z_] := coef / Length[frequencies] * Sum[amplitudes[[i]] * BF[frequencies[[i]], z],{i,1,Length[frequencies]}];
    AppendTo[images, {groups[[ig]], Import["Diagrams/"<>groups[[ig]]<>".png"], Style[CreateImageFromPalette[rgbcolors, paletterange, g, wallpaperrange, wallpaperresolution], Magnification -> magnification]}],
    {ig, 1, Length[groups]}
  ];
  Return[images]
];

(* Routine to create morphing pictures with symmetry of given wallpaper groups
   It actually returns an array of triples 
   {"name of the group", "picture representing the primitive cell", "morphing picture"} *)
MorphTwoPatternsOfGivenSymmetry[groups_, latticeangle_, latticelength_, frequencies1_, amplitudes1_, frequencies2_, amplitudes2_, palettefile_, paletterange_, wallpaperrange_, wallpaperresolution_, magnification_] := Module[
  {rgbcolors, images, wave, wave3, wave4, i, ig, a, b, \[Alpha], X, Y, BF, coef, L, cL, g},
  rgbcolors = Import[palettefile, "RGBColorArray"];
  images = {};
  wave[n_, z_] := Exp[2 * \[Pi] * I * (n[[1]] * X[z] + n[[2]] * Y[z])];
  wave3[n_, z_] := wave[n, z] + wave[{-n[[1]]-n[[2]], n[[1]]}, z] + wave[{n[[2]], -n[[1]]-n[[2]]}, z];
  wave4[n_, z_] := wave[n, z] + wave[{n[[2]], -n[[1]]}, z] + wave[-n, z] + wave[{-n[[2]], n[[1]]}, z];
  Do[
    {a, b} = latticelength;
    \[Alpha] = latticeangle;
    Switch[
      groups[[ig]],
      "p1",
      X[z_] := (Re[z] - Im[z] * Cot[\[Alpha]]) / a;
      Y[z_] := Im[z] / (b * Sin[\[Alpha]]);
      BF[n_, z_] := wave[n, z];
      coef = 1,
      "p2",
      X[z_] := (Re[z] - Im[z] * Cot[\[Alpha]]) / a;
      Y[z_] := Im[z] / (b * Sin[\[Alpha]]);
      BF[n_, z_] := wave[n, z] + wave[-n, z];
      coef = 1,
      "cm",
      a = 2 * latticelength[[1]] * Cos[\[Alpha]/2];
      b = latticelength[[1]] * Sin[\[Alpha]/2];
      X[z_] := Re[z] / a + Im[z] * a / (2 * b);
      Y[z_] := Re[z] / a - Im[z] * a / (2 * b);
      BF[n_, z_] := wave[n, z] + wave[{n[[2]], n[[1]]}, z];
      coef = 1/2,
      "cmm",
      a = 2 * latticelength[[1]] * Cos[\[Alpha]/2];
      b = latticelength[[1]] * Sin[\[Alpha]/2];
      X[z_] := Re[z] / a + Im[z] * a / (2 * b);
      Y[z_] := Re[z] / a - Im[z] * a / (2 * b);
      BF[n_, z_] := wave[n, z] + wave[{n[[2]], n[[1]]}, z] + wave[-n, z] + wave[{-n[[2]], -n[[1]]}, z];
      coef = 1/2,
      "pm",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + wave[{n[[1]], -n[[2]]}, z];
      coef = 1/2,
      "pg",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + (-1)^n[[1]] * wave[{n[[1]], -n[[2]]}, z];
      coef = 1/2,
      "pmm",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + wave[{n[[1]], -n[[2]]}, z] + wave[-n, z] + wave[{-n[[1]], n[[2]]}, z];
      coef = 1/2,
      "pmg",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + (-1)^n[[1]] * wave[{n[[1]], -n[[2]]}, z] + wave[-n, z] + (-1)^n[[1]] * wave[{-n[[1]], n[[2]]}, z];
      coef = 1/2,
      "pgg",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / b;
      BF[n_, z_] := wave[n, z] + (-1)^(n[[1]] + n[[2]]) * wave[{n[[1]], -n[[2]]}, z] + wave[-n, z] + (-1)^(n[[1]] + n[[2]]) * wave[{-n[[1]], n[[2]]}, z];
      coef = 1/2,
      "p4",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / a;
      BF[n_, z_] := wave4[n, z];
      coef = 1/2,
      "p4m",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / a;
      BF[n_, z_] := wave4[n, z] + wave4[{n[[1]], -n[[2]]}, z];
      coef = 1/4,
      "p4g",
      X[z_] := Re[z] / a;
      Y[z_] := Im[z] / a;
      BF[n_, z_] := wave4[n, z] + (-1)^(n[[1]] + n[[2]]) * wave4[{n[[2]], n[[1]]}, z];
      coef = 1/4,
      "p3",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z];
      coef = 1/3,
      "p31m",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z] + wave3[{n[[2]], n[[1]]}, z];
      coef = 1/4,
      "p3m1",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z] + wave3[{-n[[2]], -n[[1]]}, z];
      coef = 1/4,
      "p6",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z] + wave3[-n, z];
      coef = 1/4,
      "p6m",
      X[z_] := (Re[z] + Im[z] / Sqrt[3]) / a;
      Y[z_] := 2 * Im[z] / (a * Sqrt[3]);
      BF[n_, z_] := wave3[n, z] + wave3[{n[[2]], n[[1]]}, z] + wave3[-n, z] + wave3[{-n[[2]], -n[[1]]}, z];
      coef = 1/6
    ];
    L = wallpaperrange[[2,1]] - wallpaperrange[[1,1]];
    cL = 1/6;
    g[z_] := coef * If[
      Re[z]- wallpaperrange[[1,1]] <= cL * L, 
      Sum[amplitudes1[[i]] * BF[frequencies1[[i]], z],{i,1,Length[frequencies1]}] / Length[frequencies1],
      If[
        wallpaperrange[[2,1]] - Re[z] <= cL * L, 
        Sum[amplitudes2[[i]] * BF[frequencies2[[i]], z],{i,1,Length[frequencies2]}] / Length[frequencies2],
        (((1 - cL) * L - Re[z]) * Sum[amplitudes1[[i]] * BF[frequencies1[[i]], z],{i,1,Length[frequencies1]}] / Length[frequencies1] +
        (Re[z] - cL * L) * Sum[amplitudes2[[i]] * BF[frequencies2[[i]], z],{i,1,Length[frequencies2]}] / Length[frequencies2]) / ((1 - 2 * cL) * L)
      ]
    ];
    AppendTo[images, {groups[[ig]], Import["Diagrams/"<>groups[[ig]]<>".png"], Style[CreateImageFromPalette[rgbcolors, paletterange, g, wallpaperrange, wallpaperresolution], Magnification -> magnification]}],
    {ig, 1, Length[groups]}
  ];
  Return[images]
];

End[ ];
 
(* End `Private` Context. *)

SetAttributes[{
  CreateImageFromPalette,
  CreateRosetteByAveraging,
  CreateFriezeByCopying,
  CreateWallpaperOfGivenSymmetry,
  MorphTwoPatternsOfGivenSymmetry
},
{ Protected, ReadProtected }
];

EndPackage[ ]; (* End package Context. *)




